
;;*****************************************************************************
;;*****************************************************************************
;;  FILENAME: DTM.asm
;;   Version: 1.5, Updated on 2013/5/19 at 10:43:28
;;  Generated by PSoC Designer 5.4.2946
;;
;;  DESCRIPTION: DTMFDialer User Module software implementation file.
;;
;;  NOTE: User Module APIs conform to the fastcall convention for marshalling
;;        arguments and observe the associated "Registers are volatile" policy.
;;        This means it is the caller's responsibility to preserve any values
;;        in the X and A registers that are still needed after the API
;;        function returns. Even though some of these registers may not be modified
;;        now, there is no guarantee that will remain the case in future releases.
;;-----------------------------------------------------------------------------
;;  Copyright (c) Cypress Semiconductor 2013. All Rights Reserved.
;;*****************************************************************************
;;*****************************************************************************

include "DTM.inc"
include "m8c.inc"
include "memory.inc"

;-----------------------------------------------
;  Global Symbols
;-----------------------------------------------
export   DTM_Start
export  _DTM_Start
export   DTM_StartSamples
export  _DTM_StartSamples
export   DTM_SetPower
export  _DTM_SetPower
export   DTM_Stop
export  _DTM_Stop
export   DTM_StopSamples
export  _DTM_StopSamples
export   DTM_DialFromRam
export  _DTM_DialFromRam
export   DTM_DialFromRom
export  _DTM_DialFromRom
export   DTM_DialDigit
export  _DTM_DialDigit
export   DTM_bReadStatus
export  _DTM_bReadStatus
export  bDTM_ReadStatus   ; Do not use
export _bDTM_ReadStatus   ; Do not use
export   DTM_fState
export  _DTM_fState
export  _DTM_ISR

;export for BACKGROUND mode

IF (DTM_BACKGROUND_ENABLE)

export   DTM_pcPhoneNumber
export   DTM_iLowToneDelta
export   DTM_iHighToneDelta
export   DTM_iLowTone
export   DTM_iHighTone
export   DTM_cDigit
export   DTM_iToneCount
export   DTM_fStatus

ENDIF

;export for FOREGROUND mode

IF (DTM_FOREGROUND_ENABLE)

export ToneGen
export ToneInterval

ENDIF

;export always

export   DTM_cNextTone

;-----------------------------------------------
; Variable Allocation
;-----------------------------------------------
area InterruptRAM(RAM)

; globals for BACKGROUND mode

IF (DTM_BACKGROUND_ENABLE)

   DTM_pcPhoneNumber:   blk  2                   ;pointer to phone number
   DTM_iLowToneDelta:   blk  2                   ;delta value needed to generate a low tone
   DTM_iHighToneDelta:  blk  2                   ;delta value needed to generate a high tone
   DTM_iLowTone:        blk  2                   ;phase value for low tone
   DTM_iHighTone:       blk  2                   ;phase value for high tone
   DTM_cDigit:          blk  1                   ;The value set to the DAC
   DTM_iToneCount:      blk  2                   ;The deleay required to send a tone.
   DTM_fStatus:         blk  1                   ;Status flags

ENDIF

; global for all modes

   DTM_cNextTone:       blk  1                   ;The digit to be dialed
  _DTM_fState:
   DTM_fState:          blk  1



area UserModules(ROM,REL)

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: DTM_Start
;  FUNCTION NAME: DTM_SetPower
;
;  DESCRIPTION:
;    Applies power setting to the module's analog PSoc block.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;    A - contains the power setting.
;
;  RETURNS: none
;
;  SIDE EFFECTS:
;	 The A and X registers may be modified by this or future implementations
;    of this function.  The same is true for all RAM page pointer registers in
;    the Large Memory Model.  When necessary, it is the calling function's
;    responsibility to perserve their values across calls to fastcall16 
;    functions.
;
 DTM_Start:
_DTM_Start:
 DTM_SetPower:
_DTM_SetPower:
   RAM_PROLOGUE RAM_USE_CLASS_2
   push X          ;save X
   mov  X,SP       ;X will point at next pushed value
   and  A,03h
   push A                  ;X points at copy of A
   mov  A,reg[DTM_DACcr3]
   and  A,~03h     ;clear power bits
   or   A,[ X ]
   mov  reg[DTM_DACcr3],A
   pop  A
   pop  X
   RAM_EPILOGUE RAM_USE_CLASS_2
   ret

.ENDSECTION

.SECTION

;-----------------------------------------------------------------------------
;  FUNCTION NAME: DTM_StartSamples
;
;  DESCRIPTION:
;    Get the DTMF_Clk going so zeros can be written to the DAC
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: none
;
;    RETURNS: none
;
;  SIDE EFFECTS:
;	 The A and X registers may be modified by this or future implementations
;    of this function.  The same is true for all RAM page pointer registers in
;    the Large Memory Model.  When necessary, it is the calling function's
;    responsibility to perserve their values across calls to fastcall16 
;    functions.
;          
;    Currently only the page pointer registers listed below are modified:
;       CUR_PP
;
DTM_StartSamples:
_DTM_StartSamples:
   RAM_PROLOGUE RAM_USE_CLASS_4
   RAM_SETPAGE_CUR >DTM_cNextTone
   mov [DTM_cNextTone],0
   mov [DTM_fState],14
IF (DTM_FOREGROUND_ENABLE)
   or [DTM_fState], DTMF_DATA_INVALID
ENDIF
   or  reg[INT_MSK1],DTM_DTMFMask
   or  reg[DTM_DTMF_ClkCR0], 01h
   RAM_EPILOGUE RAM_USE_CLASS_4
   ret


.ENDSECTION

.SECTION

;-----------------------------------------------------------------------------
;  FUNCTION NAME: DTM_Stop
;
;  DESCRIPTION:
;    Disable the DTMF_Clk so no more DAC updates can be made.
;    Removes power from the module's analog PSoc block.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: none
;
;  RETURNS:  none
;
;  SIDE EFFECTS:
;    The A and X registers may be modified by this or future implementations
;    of this function.  The same is true for all RAM page pointer registers in
;    the Large Memory Model.  When necessary, it is the calling function's
;    responsibility to perserve their values across calls to fastcall16 
;    functions.
;          
;    Currently only the page pointer registers listed below are modified:
;       CUR_PP
;
;-----------------------------------------------------------------------------
 DTM_Stop:
_DTM_Stop:
   RAM_PROLOGUE RAM_USE_CLASS_4
   ;shut down the analog section last
   and reg[DTM_DACcr3], ~03h

IF (DTM_BACKGROUND_ENABLE)
   RAM_SETPAGE_CUR >DTM_fStatus
   and [DTM_fStatus], ~DTMF_DIALING
ENDIF
   RAM_EPILOGUE RAM_USE_CLASS_4
   ret


.ENDSECTION

.SECTION

;-----------------------------------------------------------------------------
;  FUNCTION NAME: DTM_StopSamples
;
;  DESCRIPTION:
;    Disable the DTMF_Clk so no more DAC updates can be made.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: none
;
;  RETURNS:  none
;
;  SIDE EFFECTS:
;	 The A and X registers may be modified by this or future implementations
;    of this function.  The same is true for all RAM page pointer registers in
;    the Large Memory Model.  When necessary, it is the calling function's
;    responsibility to perserve their values across calls to fastcall16 
;    functions.
;          
;    Currently only the page pointer registers listed below are modified:
;       CUR_PP
;
DTM_StopSamples:
_DTM_StopSamples:
   RAM_PROLOGUE RAM_USE_CLASS_4
   and reg[DTM_DTMF_ClkCR0],~01h
   M8C_DisableIntMask INT_MSK1, DTM_DTMFMask


IF (DTM_BACKGROUND_ENABLE)
   RAM_SETPAGE_CUR >DTM_fStatus
   and [DTM_fStatus], ~DTMF_DIALING
ENDIF
   RAM_EPILOGUE RAM_USE_CLASS_4
   ret


.ENDSECTION

.SECTION

;-----------------------------------------------------------------------------
;  FUNCTION NAME: DTM_DialFromRam
;
;  DESCRIPTION:
;    Starts the process to retrive a phone number from RAM and dial it
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;    A:X hold a pointer to a string in program memory.
;
;  RETURNS: none
;
;  SIDE EFFECTS:
;	 The A and X registers may be modified by this or future implementations
;    of this function.  The same is true for all RAM page pointer registers in
;    the Large Memory Model.  When necessary, it is the calling function's
;    responsibility to perserve their values across calls to fastcall16 
;    functions.
;          
;    Currently only the page pointer registers listed below are modified:
;       CUR_PP
;       IDX_PP ( If foreground is enabled )
;
;  THEORY of OPERATION or PROCEDURE:
;
;  Foreground variable locations
;
;  phone number   2      X-13
;  iLowToneDelta  2      X-11
;  iHighToneDelta 2      X-9
;  iLowTone       2      X-7
;  iHighTone      2      X-5
;  cDigit         1      X-3
;  iToneCount     2      X-2
;  fStatus        1      X-0
;  SP                    X+1
;
;
;-----------------------------------------------------------------------------

DTM_DialFromRam:
_DTM_DialFromRam:
   RAM_PROLOGUE RAM_USE_CLASS_4
   RAM_SETPAGE_CUR >DTM_fState

IF (DTM_BACKGROUND_ENABLE)

; entry point for Background/interrupt based processing
   mov [DTM_pcPhoneNumber + HighByte ],A
   mov [DTM_pcPhoneNumber + LowByte],X
   and [DTM_fStatus],~DTMF_READFRROM
   mov [DTM_fState],S_GETDIG
   or [DTM_fStatus], DTMF_DIALING

   ret
ENDIF

IF (DTM_FOREGROUND_ENABLE)

; entry point for Foreground/stack based processing
   push A
   push X
   mov X,SP
   mov A, X
   add A, 11                                               ; less the two earlier pushes
   mov X, A
   inc A                                                   ; make sure stack pointer is above data
   swap A, SP                                              ; x points at fStatus now
   or [DTM_fState], DTMF_DATA_INVALID
   ; operate through foreground loop
   jmp Next_RAM_Digit

;
; end DialFromRam
;
ENDIF


.ENDSECTION

.SECTION

;-----------------------------------------------------------------------------
;  FUNCTION NAME: DTM_DialFromRom
;
;  DESCRIPTION:
;    Starts the process to retrive a phone number from ROM and dial it
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;    A:X holds a pointer to a string in data memory.
;
;  RETURNS: none
;
;  SIDE EFFECTS:
;    The A and X registers may be modified by this or future implementations
;    of this function.  The same is true for all RAM page pointer registers in
;    the Large Memory Model.  When necessary, it is the calling function's
;    responsibility to perserve their values across calls to fastcall16 
;    functions.
;          
;    Currently only the page pointer registers listed below are modified:
;       CUR_PP
;
;  THEORY of OPERATION or PROCEDURE:
;
;  Foreground variable locations
;
;   phone number   2      X-13
;   iLowToneDelta  2      X-11
;   iHighToneDelta 2      X-9
;   iLowTone       2      X-7
;   iHighTone      2      X-5
;   cDigit         1      X-3
;   iToneCount     2      X-2
;   fStatus        1      X-0
;   SP                    X+1
;
;-----------------------------------------------------------------------------

DTM_DialFromRom:
_DTM_DialFromRom:
   RAM_PROLOGUE RAM_USE_CLASS_4
   RAM_SETPAGE_CUR >DTM_fState
IF (DTM_BACKGROUND_ENABLE)

   mov [DTM_pcPhoneNumber + HighByte ],A
   mov [DTM_pcPhoneNumber + LowByte],X
   or  [DTM_fStatus],DTMF_READFRROM
   mov [DTM_fState],S_GETDIG
   or [DTM_fStatus], DTMF_DIALING
   RAM_EPILOGUE RAM_USE_CLASS_4
   ret
ENDIF

IF (DTM_FOREGROUND_ENABLE)
; entry point for Foreground/stack based processing

   push A
   push X
   mov X,SP
   mov A, X
   add A, 11                                               ; less the two earlier pushes
   mov X, A
   inc A                                                   ; make sure stack pointer is above data
   swap A, SP                                              ; x points at fStatus now
   or [DTM_fState], DTMF_DATA_INVALID
   ; operate through foreground loop
   RAM_EPILOGUE RAM_USE_CLASS_4
   jmp Next_ROM_Digit

; end DialFromRom

ENDIF


.ENDSECTION

.SECTION

;-----------------------------------------------------------------------------
;  FUNCTION NAME: DTM_DialDigit
;
;  DESCRIPTION:
;    Starts the process to retrive a phone number from ROM and dial it
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;    A holds an ascii digit.  Null terminates after configured delay.
;
;  RETURNS: none
;
;  SIDE EFFECTS:
;    The A and X registers may be modified by this or future implementations
;    of this function.  The same is true for all RAM page pointer registers in
;    the Large Memory Model.  When necessary, it is the calling function's
;    responsibility to perserve their values across calls to fastcall16 
;    functions.
;          
;    Currently only the page pointer registers listed below are modified:
;       CUR_PP (If Background is enabled)
;
DTM_DialDigit:
_DTM_DialDigit:

IF (DTM_BACKGROUND_ENABLE)

   RAM_PROLOGUE RAM_USE_CLASS_4
   RAM_SETPAGE_CUR >DTM_fState

   tst [DTM_fStatus], DTMF_CONTINUOUS_TONE
   jz setTone
   and [DTM_fStatus], ~DTMF_CONTINUOUS_TONE                ; clear continuous tone and wait
waitOldTone:
   tst [DTM_fStatus], DTMF_DIALING
   jnz waitOldTone
   add A,0
   jnz setTone
   ret                                                     ; if a null was passed into routine
setTone:
   mov [DTM_cDigit ],A
   ; assume everything is running and ready to go
   or [DTM_fStatus], DTMF_CONTINUOUS_TONE | DTMF_DIALING
   mov [DTM_fState],S_DECODE
   ; set to state = 3 but set status to loop through state 10 which will bypass the decrement
   ; of toneCount.
   ; stop when a null is passed as a digit.
   RAM_EPILOGUE RAM_USE_CLASS_4

ENDIF

   ret
.ENDSECTION

.SECTION

;-----------------------------------------------------------------------------
;  FUNCTION NAME: DTM_bReadStatus
;
;  DESCRIPTION:
;    Return the fStatus byte in the accumulator
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: none
;
;  RETURNS:
;    fStatus byte in the accumulator
;
;  SIDE EFFECTS:
;    The A and X registers may be modified by this or future implementations
;    of this function.  The same is true for all RAM page pointer registers in
;    the Large Memory Model.  When necessary, it is the calling function's
;    responsibility to perserve their values across calls to fastcall16 
;    functions.
;          
;    Currently only the page pointer registers listed below are modified:
;       CUR_PP ( If background is enabled )
;
;-----------------------------------------------------------------------------
 bDTM_ReadStatus:
_bDTM_ReadStatus:
 DTM_bReadStatus:
_DTM_bReadStatus:
   RAM_PROLOGUE RAM_USE_CLASS_4
IF (DTM_BACKGROUND_ENABLE)
   RAM_SETPAGE_CUR >DTM_fState
   mov A, [DTM_fStatus]
ELSE
   mov A, ~DTMF_DIALING                                    ;by definition if you call this with foreground enabled
                                                           ;the dialer my be inactive.
ENDIF
   RAM_EPILOGUE RAM_USE_CLASS_4
   ret


.ENDSECTION

.SECTION

IF (DTM_FOREGROUND_ENABLE)
;;*********************************************************************
;;
;;    COMMON ENTRY POINT for foreground DialFromRam and DialFromRom routines
;;      to enter the state machine and generate DTMF tones.
;;
;;   DTM_DialFromRam
;;   DTM_DialFromRom
;;
;;   phone number   2      X-13
;;   iLowToneDelta  2      X-11
;;   iHighToneDelta 2      X-9
;;   iLowTone       2      X-7
;;   iHighTone      2      X-5
;;   cDigit         1      X-3
;;   iToneCount     2      X-2
;;   fStatus        1 ???  X-0
;;   SP                    X+1
;;
;;
;;
;;*********************************************************************
pcPhoneNumber:   equ -13   ;2 bytes
iLowToneDelta:   equ -11   ;2 bytes
iHighToneDelta:  equ -9    ;2 bytes
iLowTone:        equ -7    ;2 byte
iHighTone:       equ -5    ;2 byte
cDigit:          equ -3    ;1 byte
iToneCount:      equ -2    ;2 bytes
fStatus:         equ  0    ;1 bytes
; SP             X+1


wait_for_TICK:
   RAM_PROLOGUE RAM_USE_CLASS_4
   RAM_SETPAGE_CUR >DTM_fState
.wait_tick:
   tst [DTM_fState], DTMF_INTERRUPT_TICK
   jz .wait_tick
   and [DTM_fState], ~DTMF_INTERRUPT_TICK
   mov A, [DTM_fState]
   and A, ~DTMF_INTERRUPT_TICK                             ; cancel tick prior to jump
   
   RAM_EPILOGUE RAM_USE_CLASS_4
   jacc FGStateTable
FGStateTable:
   ;tone generation is called by the interrupt, so when the digit
   ;is all decoded and setup wait for
   jmp wait_for_TICK           ;update the PWM DAC
   jmp wait_for_TICK           ;pause state sends 0 out to PWM DAC
   jmp GetNextDigit

;*********************************************************************
IdleState:
   ret
;*********************************************************************
GetNextDigit: ;(get the next digit from Ram or Rom)
   RAM_PROLOGUE RAM_USE_CLASS_2
   tst [X + fStatus ], DTMF_READFRROM                      ; test for dial from ROM flag
   RAM_EPILOGUE RAM_USE_CLASS_2
   jz Next_RAM_Digit
   ;fall through to Next_ROM_Digit
   ;
   ;(Number is in ROM)
   ;
Next_ROM_Digit:
   RAM_PROLOGUE RAM_USE_CLASS_2
   mov A, [X + (pcPhoneNumber + HighByte)]
   push X
   mov X,[X + (pcPhoneNumber + LowByte)]
   romx
   pop X
   mov [X + cDigit],A
   mov [X + fStatus ], DTMF_READFRROM                      ; set dial from ROM flag
   ;
   ; pend here based on DTMF_INTERRUPT_TICK
   RAM_EPILOGUE RAM_USE_CLASS_2
   jmp ProcessDigit                                        ; Jmp to common entry point below
   ;ret will happen somewhere below

   ;
   ;(Number is in RAM)
   ;
Next_RAM_Digit:
   RAM_PROLOGUE RAM_USE_CLASS_3
   push X
   RAM_SETPAGE_IDX2STK
   mov A,[X + (pcPhoneNumber + HighByte)]
   mov X,[X + (pcPhoneNumber + LowByte)]
   RAM_SETPAGE_IDX A
   mov A,[X]
   pop X
   RAM_SETPAGE_IDX2STK
   mov [X + cDigit],A
   mov [X + fStatus ], 0                                   ; clear dial from ROM flag
   RAM_EPILOGUE RAM_USE_CLASS_3
   ;
   ; pend here based on DTMF_INTERRUPT_TICK
   ;jmp ProcessDigit                                       ; Jmp to common entry point below
   ;ret will happen somewhere below                        ; Jmp stmt left in for clarity


ProcessDigit:
   RAM_PROLOGUE RAM_USE_CLASS_3
   ;
   ; increment pointers to  [cDigit] for next tone. Examine current data,
   ; if the value is a 0/null this is assumed to be the string terminator.
   ;
   inc [X +(pcPhoneNumber + LowByte)]                      ;increment pointer
   adc [X + (pcPhoneNumber + HighByte)],0
   cmp [X + cDigit],00h
   jz endPNString
   ;(charcter is valid)
   RAM_EPILOGUE RAM_USE_CLASS_3
   jmp DecodeDigit

endPNString:;(end of phonenumber)
   ;
   ;at end of number, fix stack and return
   ;
   mov X,SP
   mov A, X
   add A, -13                                              ; less the two earlier pushes
   dec A
   swap A, SP

   ;and [X + fStatus], ~DTMF_DIALING
   RAM_EPILOGUE RAM_USE_CLASS_3
   ret

;*********************************************************************
DecodeDigit:;( decode the tone)
   RAM_PROLOGUE RAM_USE_CLASS_4
   RAM_EPILOGUE RAM_USE_CLASS_2
   RAM_SETPAGE_CUR >DTM_fState
   cmp  [X + cDigit],2ch
   jnz notComma
   ;(If cDigit is a comma)
   mov [X + (iToneCount + LowByte)],(<iNoToneLength)
   mov [X + (iToneCount + HighByte)],(>iNoToneLength)
   and [DTM_fState],NOSTATE
   or  [DTM_fState],S_TONESPACE                            ;set next state ToneInterval/go to pause state
   RAM_EPILOGUE RAM_USE_CLASS_2
   RAM_EPILOGUE RAM_USE_CLASS_4
   jmp wait_for_TICK

 notComma:
   sub [X + cDigit],20h                                    ;shift data by 32 values
   jnc aboveLowLim
   ; character is out of legal range                       ;(the lowest 32d/20h chacters are ignored)
   and [DTM_fState],NOSTATE
   or  [DTM_fState],S_DECODE                               ;set next state DecodeDigit & get the next digit
   tst [X + fStatus ], DTMF_READFRROM                      ; test for dial from ROM flag
   RAM_EPILOGUE RAM_USE_CLASS_2
   RAM_EPILOGUE RAM_USE_CLASS_4
   jz Next_RAM_Digit
   jmp Next_ROM_Digit

 aboveLowLim:
   cmp [X + cDigit],40h                                    ;is [cDigit] larger than 40h?
   jc  notLowerCase                                        ;[cDigit] smaller than 40h: jump
   ;(lower case alphas [cDigit] which means the character started out at 60h or more)
   sub [X + cDigit],20h                                    ;make upper case alpha
 notLowerCase:
   cmp [X + cDigit],25h
   RAM_EPILOGUE RAM_USE_CLASS_2
   RAM_EPILOGUE RAM_USE_CLASS_4
   jc  ConvertTone                                         ;only alphas D or smaller are valid
   ;(Digit > D)
   and [DTM_fState],NOSTATE
   or [DTM_fState],S_DECODE                                ;set next state DecodeDigit & get the next digit
   tst [X + fStatus ], DTMF_READFRROM                      ; test for dial from ROM flag
   RAM_EPILOGUE RAM_USE_CLASS_2
   RAM_EPILOGUE RAM_USE_CLASS_4
   jz Next_RAM_Digit
   jmp Next_ROM_Digit
;;*********************************************************************
;; At this point character is in a valid range from 03-24 hex
;; remember 20h was already subtracted!
;; valid chars # 03h
;;             * 0Ah
;;           0-9 10h-19h
;;           A-D 21h-24h
;; Attempting to use a table look up for the 16 characters lower nibble will have 0-3 corresponding
;; to the 0-3 low tones.
;; higher nibble will have 0-3 corresponding to the 0-3 high tones.
;; Will just create an index into a table to come up with a tone pair and use the tone pair to
;; load tone step values.
;;   hitone    0     1     2     3
;;lotone   | 1209  1336  1477 1633
;;    -----+------------------------
;;  0  697 |  1 0:0 2 1:0 3 2:0 A 3:0
;;  1  770 |  4 0:1 5 1:1 6 2:1 B 3:1
;;  2  852 |  7 0:2 8 1:2 9 2:2 C 3:2
;;  3  941 |  * 0:3 0 1:3 # 2:3 D 3:3
;;
;;*********************************************************************
ConvertTone:
   RAM_PROLOGUE RAM_USE_CLASS_4
   RAM_PROLOGUE RAM_USE_CLASS_2
   RAM_SETPAGE_CUR >DTM_fState
   mov A,[X + cDigit]                                      ;convert cDigit to point to 1 bit values in
   index ToneTable
   push A
   and A, 0xCC
   jz ProcTone                                             ;bits in the upper positions are invalid value --skip it.
   ;digit was invalid
   pop A                                                   ;Clean up the stack
   and [DTM_fState],NOSTATE
   or  [DTM_fState],S_DECODE                               ;set next state DecodeDigit & get the next digit
   tst [X + fStatus ], DTMF_READFRROM                      ; test for dial from ROM flag
   RAM_EPILOGUE RAM_USE_CLASS_2
   RAM_EPILOGUE RAM_USE_CLASS_4
   jz Next_RAM_Digit
   jmp Next_ROM_Digit

   ;
   ;  process the low tone value
   ;
ProcTone:
   pop A
   push A                                                  ; Store A, content has nibbles corresponding to tones
   and A,03h                                               ; Mask off the lower two bits (value will be between 0 and 3)
   asl A                                                   ; multiply by 2
   push A
   index LowTone
   mov [X + iLowToneDelta + HighByte],A
   pop A
   index (LowTone + 1)
   mov [X + iLowToneDelta + LowByte],A
   pop A
;
;  process the high tone
;
   asr A                                                      ; shift xxBB00xx to 000xxBB0 3 places
   asr A
   asr A
   push A
   index HighTone
   mov [X + iHighToneDelta +HighByte],A
   pop A
   index (HighTone + 1)
   mov [X + iHighToneDelta +LowByte],A
   and [DTM_fState],NOSTATE
   or  [DTM_fState],S_LAUNCH                               ;set next state LaunchDAC/set up for action
   ;drop through
;;*********************************************************************
LaunchDAC:;(Launch DAC)
   mov [X + (iToneCount + LowByte)], (<iToneOnLength)
   mov [X + (iToneCount + HighByte)],(>iToneOnLength)
   mov [X + (iLowTone + HighByte)],0
   mov [X + (iLowTone + LowByte)],80h
   mov [X + (iHighTone + HighByte)],0
   mov [X + (iHighTone + LowByte)],80h
   mov [DTM_cNextTone],0
   and [DTM_fState], ~DTMF_DATA_INVALID
   and [DTM_fState],NOSTATE
   or  [DTM_fState],S_TONE                                 ;next state ToneGen/put the tones
   ;
   ; instead of just starting the DAC, go back and wait for the next tick to avoid a short first tick period
   ;
   ; don't know if a tick is pending, but it shouldn't matter
   RAM_EPILOGUE RAM_USE_CLASS_2
   RAM_EPILOGUE RAM_USE_CLASS_4
   jmp wait_for_TICK


;;*********************************************************************
;;  ToneGen and ToneInterval are the two states that are only entered when a TICK happens
;;    They each process data in preparation for the next TICK so that it's ready
;;    as soon as the interrupt fires.
;;*********************************************************************
ToneGen: ; This is the state were the tone is updated to the PWM
         ; how many cycles
   dec [X + (iToneCount + LowByte)]                            ;decrement pause count
   sbb [X + (iToneCount + HighByte)],00h                       ;subtract the carry if there was one
   jc endTone
   ;(If tone is still to be output)
   ;calculate new cNextTone
   ;generate new phase values for LowTone & HighTone
   mov A,[X + iLowToneDelta + LowByte]
   add [X + iLowTone + LowByte],A
   mov A,[X + iLowToneDelta + HighByte]
   adc [X + iLowTone + HighByte],A
   mov A,[X + iHighToneDelta + LowByte]
   add [X + iHighTone + LowByte],A
   mov A,[X + iHighToneDelta + HighByte]
   adc [X + iHighTone + HighByte],A

   ;now the pointers to the sinetable are set for the next
   ;table lookup
   mov   A,[X + iLowTone + HighByte]                       ;place 3/4 of LowTone table value in NextTone
   index SineTable
   mov   [DTM_cNextTone],A
   asr   A
   add   [DTM_cNextTone],A
   inc   [DTM_cNextTone]                                   ;Now have 1.5 X SineTable value
   asr   [DTM_cNextTone]                                   ;div by 2 for 3/4

   mov   A,[X + iHighTone + HighByte]                      ;Now have 1.5 X SineTable value
   index SineTable
   add   [DTM_cNextTone],A

   add   [DTM_cNextTone],02h                               ;Dac is 6 bit this amounts to rounding up
   asr   [DTM_cNextTone]                                   ; the fraction before converting to 6 bit
   asr   [DTM_cNextTone]                                   ;convert to 6 bit with shifting.
   jmp endToneGen                                          ;end of process for this tick
 endTone:;(Stop outputing tone)
   mov [X + (iToneCount + LowByte)],(<iToneOffLength)
   mov [X + (iToneCount + HighByte)],(>iToneOffLength)
   and [DTM_fState],NOSTATE
   or  [DTM_fState], S_TONESPACE                           ;set next state to pause/ToneInterval
   asr  [DTM_cNextTone]   ;move the DAC back to zero
   tst [DTM_cNextTone],80h
   jz endToneGen
   ;(less than 128)
   inc [DTM_cNextTone]
 endToneGen:
 ;return from interrupt that jumped here.
 pop A
  reti
   ;jmp wait_for_TICK


;;*********************************************************************
ToneInterval:                                              ; (Pause State)
;EndContTone:                                              ; (Pause State prior to ending CONTONUOUS_TONE)
   ;
   ; set the DAC to zero (analog gnd) and wait for the  period defined by iToneCount
   ;
   asr  [DTM_cNextTone]                                    ;move the DAC back to zero
   tst [DTM_cNextTone],80h
   jz positive
   ;(less than 0)
   inc [DTM_cNextTone]
 positive:
   dec [X + (iToneCount + LowByte)]                          ;decrement pause count
   sbb [X + (iToneCount + HighByte)],00h
   jnc contInterval
   and [DTM_fState],NOSTATE
   or  [DTM_fState], S_GETDIG                              ;set next state to DecodeDigit & get next dig.
 contInterval:
   ;return from interrupt that jumped here.
   pop A
   reti


ENDIF //endif bracket for whole foreground routine



;;area UserModules(ROM,REL)
;;------------------------------------------------------------------
;; FUNCTION NAME: _DTM_ISR:
;;      This routine manages a state machine that fetches a character to dial
;;      and determines tone pairs.   Most of the code in this file supports the
;;      BACKGROUND mode of operation which trades use of more RAM global variabls for
;;      fully interrupt based operation while other functions may be running.
;;
;;  INPUTS:  uses the state variable: DTM_fState and 10 other variables
;;           defined in the DTM.asm file.
;;
;;           State values are defined in the DTM.inc file.
;;
;;  OUTPUTS: None.
;;
;;------------------------------------------------------------------

_DTM_ISR:
   push A
   RAM_X_POINTS_TO_STACKPAGE                                  ;Set the page mode to index page points to stack page
IF (DTM_FOREGROUND_ENABLE)
   tst [DTM_fState], DTMF_DATA_INVALID                     ;flag set if data is in mid-calculation or has not been setup
   jnz SkipState
ENDIF
   mov  A,[DTM_cNextTone]
   cmp A,80h
   jnc  elseif1
   xor A,80h
   jmp endif1
 elseif1:;(negative value)
   xor A,5fh
   inc A
 endif1:
   ;or A, 0x40h  ;will invert the phase of the output clock
   M8C_Stall
   mov  reg[DTM_DACcr0], A
   M8C_Unstall

   mov  A,[DTM_fState]

IF (DTM_FOREGROUND_ENABLE)
   ;clear the tick bit
   and A, ~DTMF_INTERRUPT_TICK
   cmp A, S_TONESPACE +1
   jnc SkipState                                           ;jump if state other than 1st two states
ENDIF

   jacc StateTable
StateTable:
   jmp ToneGen                                             ;(S_TONE)update the PWM DAC
   jmp ToneInterval                                        ;(S_TONESPACE)pause state sends 0 out to PWM DAC
   ; should never get to the following instructions if FOREGROUND mode is enabled
IF (DTM_BACKGROUND_ENABLE)
   jmp GetDigit                                            ;(S_GETDIG)
   jmp DecodeDigit                                         ;(S_DECODE)
   jmp LaunchDAC                                           ;(S_LAUNCH)
   jmp ContinuousTone                                      ;(S_CONTTONE)
   jmp EndContTone                                         ;(S_ENDCONT)end the continuous tone
   jmp IdleState                                           ;(S_IDLE)do nothing
ENDIF

SkipState:
IF (DTM_FOREGROUND_ENABLE)
   or [DTM_fState], DTMF_INTERRUPT_TICK
ENDIF
   pop A
   reti

IF (DTM_BACKGROUND_ENABLE)
;----------------------------------------------------------------
IdleState:;(Do Nothing)
   pop A
   reti
;----------------------------------------------------------------
ContinuousTone:
   tst [DTM_fStatus], DTMF_CONTINUOUS_TONE
   jz endTone
   jmp toneOn
ToneGen:  ; This is the state were the tone is updated to the PWM
          ; (186 cpu cycles in ToneGen: from here
   dec [(DTM_iToneCount + LowByte)]                        ;decrement pause count
   sbb [(DTM_iToneCount + HighByte)],00h                   ;subtract the carry if there was one
   jc endTone
 toneOn:;(If tone is still to be output)
   ;calculate new cNextTone
   ;generate new phase values for LowTone & HighTone
   mov A,[DTM_iLowToneDelta + LowByte]
   add [DTM_iLowTone + LowByte],A
   mov A,[DTM_iLowToneDelta + HighByte]
   adc [DTM_iLowTone + HighByte],A
   mov A,[DTM_iHighToneDelta + LowByte]
   add [DTM_iHighTone + LowByte],A
   mov A,[DTM_iHighToneDelta + HighByte]
   adc [DTM_iHighTone + HighByte],A

   ;now the pointers to the sinetable are set for the next
   ;table lookup
   mov   A,[DTM_iLowTone + HighByte]                       ;place 3/4 of LowTone table value in NextTone
   index SineTable
   mov   [DTM_cNextTone],A
   asr   A
   add   [DTM_cNextTone],A
   inc   [DTM_cNextTone]                                   ;Now have 1.5 X SineTable value
   asr   [DTM_cNextTone]                                   ;div by 2 for 3/4

   mov   A,[DTM_iHighTone + HighByte]                      ;Now have 1.5 X SineTable value
   index SineTable
   add   [DTM_cNextTone],A

   add   [DTM_cNextTone],02h                               ;Dac is 6 bit this amounts to rounding up
   asr   [DTM_cNextTone]                                   ; the fraction before converting to 6 bit
   asr   [DTM_cNextTone]                                   ;convert to 6 bit with shifting.

   pop   A
   reti
 endTone:;(Stop outputing tone)
   mov [(DTM_iToneCount + LowByte)],(<iToneOffLength)
   mov [(DTM_iToneCount + HighByte)],(>iToneOffLength)
   cmp [DTM_fState], S_CONTTONE
   jz endConTone
   mov [DTM_fState], S_TONESPACE                           ;set next state to pause/ToneInterval
   jmp endNormTone
 endConTone:
   mov [DTM_fState], S_ENDCONT                             ;set next state to pause/EndContTone
 endNormTone:
   asr  [DTM_cNextTone]                                    ;move the DAC back to zero
   tst [DTM_cNextTone],80h
   jz positive1DAC
   ;(DAC negative)
   inc [DTM_cNextTone]
 positive1DAC:
   pop A
   reti

;-----------------------------------------------------------------------------
ToneInterval:                                              ; (Pause State)
EndContTone:                                               ; (Pause State prior to ending CONTONUOUS_TONE)
   ;
   ; set the DAC to zero (analog gnd) and wait for the  period defined by iToneCount
   ;
   asr  [DTM_cNextTone]                                    ;move the DAC back to zero
   tst [DTM_cNextTone],80h
   jz positive2DAC
 ;(less than 0)
   inc [DTM_cNextTone]
 positive2DAC:
   dec [(DTM_iToneCount + LowByte)]                        ;decrement pause count
   sbb [(DTM_iToneCount + HighByte)],00h
   jnc endStToneInt
   cmp [DTM_fState], S_ENDCONT                             ;end of CONTINUOUS_TONE no more digits to proc.
   jz clrContToneSt
   mov [DTM_fState], S_GETDIG                              ;set next state to GetDigit/get next dig.
   jmp endStToneInt
 clrContToneSt:
   and [DTM_fStatus], ~(DTMF_CONTINUOUS_TONE | DTMF_DIALING)
   mov [DTM_fState], S_IDLE                                ;set next state to IdleState/do nothing.
 endStToneInt:
   pop A
   reti
;--------------------------------------------------------------------------
GetDigit: ;(Get cDigit State)
   ;
   ; when state is completed [cDigit] will contain the ascii digit to be operated on
   ; if the value is a 0/null this is assumed to be the string terminator.
   ;
   push X
   tst [DTM_fStatus],DTMF_READFRROM
   jz ramDig
   ;
   ;(Number is in ROM)
   ;
   mov A,[(DTM_pcPhoneNumber + HighByte)]
   mov X,[(DTM_pcPhoneNumber + LowByte)]
   romx
   mov [DTM_cDigit],A
   jmp digitFound
   ;
   ;(Number is in RAM)
   ;
 ramDig:
   IF SYSTEM_LARGE_MEMORY_MODEL
   REG_PRESERVE IDX_PP
   ENDIF
   
   mov X,[(DTM_pcPhoneNumber + LowByte)]
   mov A,[(DTM_pcPhoneNumber + HighByte)]
   RAM_SETPAGE_IDX A
   RAM_CHANGE_PAGE_MODE 10h
   mov A,[X]
   RAM_CHANGE_PAGE_MODE 01h
   mov [DTM_cDigit],A
   
   IF SYSTEM_LARGE_MEMORY_MODEL
   REG_RESTORE IDX_PP
   ENDIF

 digitFound:
   pop X
   pop A
   inc [(DTM_pcPhoneNumber + LowByte)]                     ;increment pointer
   adc [(DTM_pcPhoneNumber + HighByte)],0
   cmp [DTM_cDigit],00h
   jz endPhNum
 ;(charcter is valid)
   mov [DTM_fState],S_DECODE                               ;set next state to DecodeDigit/decode this digit
   reti
 endPhNum:;(end of phonenumber)
   mov [DTM_fState],S_IDLE                                 ;set next state IdleState/do nothing -- process done
   and [DTM_fStatus], ~DTMF_DIALING
   reti

;------------------------------------------------------------------
;
DecodeDigit: ;( decode the tone)
   cmp  [DTM_cDigit],2ch
   jnz notComma
   ;(If cDigit is a comma)
   mov [(DTM_iToneCount + LowByte)],(<iNoToneLength)
   mov [(DTM_iToneCount + HighByte)],(>iNoToneLength)
   mov [DTM_fState],S_TONESPACE                            ;set next state ToneInterval/go to pause state
   pop A
   reti
 notComma:
   sub [DTM_cDigit],20h                                    ;shift data by 32 values
   jnc aboveLowLim
   ;too small                                              ;(the lowest 32d/20h chacters are ignored)
   mov [DTM_fState],S_GETDIG                               ;set next state GetDigit/get the next digit
   pop A
   reti
 aboveLowLim:
   cmp [DTM_cDigit],40h                                    ;is [cDigit] larger than 40h?
   jc  notLowerCase                                             ;[cDigit] smaller than 40h: jump
   ;(lower case alphas [cDigit] which means the character started out at 60h or more)
   sub [DTM_cDigit],20h                                    ;make upper case alpha
 notLowerCase:
   cmp [DTM_cDigit],25h
   jc  ConvertTone                                             ;only alphas D or smaller are valid
   ;(Digit > D)
   mov [DTM_fState],S_GETDIG                               ;set next state GetDigit/get the next digit
   pop A
   reti

;------------------------------------------------------------------
; At this point character is in a valid range from 03-24 hex
; remember 20h was already subtracted!
; valid chars # 03h
;             * 0Ah
;           0-9 10h-19h
;           A-D 21h-24h
; Attempting to use a table look up for the 16 characters lower nibble will have 0-3 corresponding
; to the 0-3 low tones.
; higher nibble will have 0-3 corresponding to the 0-3 high tones.
; Will just create an index into a table to come up with a tone pair and use the tone pair to
; load tone step values.
;   hitone    0     1     2     3
;lotone   | 1209  1336  1477 1633
;    -----+------------------------
;  0  697 |  1 0:0 2 1:0 3 2:0 A 3:0
;  1  770 |  4 0:1 5 1:1 6 2:1 B 3:1
;  2  852 |  7 0:2 8 1:2 9 2:2 C 3:2
;  3  941 |  * 0:3 0 1:3 # 2:3 D 3:3

;-------------------------------------------------------------------
 ConvertTone:
   mov A,[DTM_cDigit]                                      ;convert cDigit to point to 1 bit values in
   index ToneTable
   push A
   and A, 0xCC
   jz ProcTone                                             ;bits in the upper positions are invalid value --skip it.
   ;digit was invalid
   mov [DTM_fState],S_GETDIG                               ;set next state GetDigit/get the next digit
   pop A
   pop A
   reti
   ;
   ;  process the low tone value
   ;
 ProcTone:
   pop A
   push X
   push A
   and A,03h                                               ; Mask off the lower two bits (value will be between 0 and 3)
   asl A                                                   ; multiply by 2
   mov X,A
   index LowTone
   mov [DTM_iLowToneDelta + HighByte],A
   mov A,X
   index (LowTone + 1)
   mov [DTM_iLowToneDelta + LowByte],A
   pop A
;
;  process the high tone
;
   asr A                                                   ; shift xxBB00xx to 000xxBB0 3 places
   asr A
   asr A
   mov X,A                                                 ; get the high nibble into X
   index HighTone
   mov [DTM_iHighToneDelta +HighByte],A
   mov A,X
   index (HighTone + 1)
   mov [DTM_iHighToneDelta +LowByte],A
   pop X
   pop A
   mov [DTM_fState],S_LAUNCH                               ;set next state LaunchDAC/set up for action
   reti

;-----------------------------------------------------------------------------------
LaunchDAC:;(Launch DAC)
   mov [(DTM_iToneCount + LowByte)], (<iToneOnLength)
   mov [(DTM_iToneCount + HighByte)],(>iToneOnLength)
   mov [(DTM_iLowTone + HighByte)],0
   mov [(DTM_iLowTone + LowByte)],80h
   mov [(DTM_iHighTone + HighByte)],0
   mov [(DTM_iHighTone + LowByte)],80h
   mov [ DTM_cNextTone],0
   tst [DTM_fStatus], DTMF_CONTINUOUS_TONE
   jnz setContuousTone
   ;setToneGen
   mov [DTM_fState],S_TONE                                 ;next state ToneGen/put the tones
   jmp endLaunchDAC
setContuousTone:
   mov [DTM_fState],S_CONTTONE                             ;next state ContuousTone/put the tone until stopped
endLaunchDAC:
   pop A
   reti


ENDIF  ;the whole set of state other than the basic interrupt

.ENDSECTION
;-----------------------------------------------------------------------------------
; table of tone pairs indexed by character after checks in state 4 (subtract 20 from ascii value).
;   hitone    0     1     2     3
;lotone   | 1209  1336  1477 1633
;    -----+------------------------
;  0  697 |  1 0:0 2 1:0 3 2:0 A 3:0
;  1  770 |  4 0:1 5 1:1 6 2:1 B 3:1
;  2  852 |  7 0:2 8 1:2 9 2:2 C 3:2
;  3  941 |  * 0:3 0 1:3 # 2:3 D 3:3
;-----------------------------------------------------------------------------------
.LITERAL
ToneTable::
   db 0xff ; 00
   db 0xff ; 01
   db 0xff ; 02
   db 0x23 ;#
   db 0xff ; 04
   db 0xff ; 05
   db 0xff ; 06
   db 0xff ; 07
   db 0xff ; 08
   db 0xff ; 09
   db 0x03 ;*
   db 0xff ; 0b
   db 0xff ; 0c
   db 0xff ; 0d
   db 0xff ; 0e
   db 0xff ; 0f
   db 0x13 ; 10
   db 0x00 ;1 11
   db 0x10 ;2 12
   db 0x20 ;3 13
   db 0x01 ;4 14
   db 0x11 ;5 15
   db 0x21 ;6 16
   db 0x02 ;7 17
   db 0x12 ;8 18
   db 0x22 ;9 19
   db 0xff ; 1a
   db 0xff ; 1b
   db 0xff ; 1c
   db 0xff ; 1d
   db 0xff ; 1e
   db 0xff ; 1f
   db 0xff ; 20
   db 0x30 ;A 21
   db 0x31 ;B 22
   db 0x32 ;C 23
   db 0x33 ;D 24

LowTone::
   dw (((697*65536)+(DTMFClkFreq/2))/DTMFClkFreq)       ;  697Hz  05b6h
   dw (((770*65536)+(DTMFClkFreq/2))/DTMFClkFreq)       ;  770Hz  064fh
   dw (((852*65536)+(DTMFClkFreq/2))/DTMFClkFreq)       ;  852Hz  06fbh
   dw (((941*65536)+(DTMFClkFreq/2))/DTMFClkFreq)       ;  941Hz  07b5h
HighTone::
   dw (((1209*65536)+(DTMFClkFreq/2))/DTMFClkFreq)      ; 1209Hz  09e7h
   dw (((1336*65536)+(DTMFClkFreq/2))/DTMFClkFreq)      ; 1336Hz  0af2h
   dw (((1477*65536)+(DTMFClkFreq/2))/DTMFClkFreq)      ; 1477Hz  0c19h
   dw (((1633*65536)+(DTMFClkFreq/2))/DTMFClkFreq)      ; 1633Hz  0d61h
SineTable::
   db   0,  2,  3,  5,  7,  9, 10, 12, 14, 16, 17, 19, 21, 22, 24, 26 ;0
   db  27, 29, 30, 32, 34, 35, 37, 38, 40, 41, 42, 44, 45, 47, 48, 49 ;10
   db  50, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 64, 65 ;20
   db  66, 66, 67, 68, 68, 69, 69, 69, 70, 70, 70, 71, 71, 71, 71, 71 ;30
   db  71, 71, 71, 71, 71, 71, 70, 70, 70, 69, 69, 69, 68, 68, 67, 66 ;40
   db  66, 65, 64, 64, 63, 62, 61, 60, 59, 58, 57, 56, 55, 54, 53, 52 ;50
   db  50, 49, 48, 47, 45, 44, 42, 41, 40, 38, 37, 35, 34, 32, 30, 29 ;60
   db  27, 26, 24, 22, 21, 19, 17, 16, 14, 12, 10,  9,  7,  5,  3,  2 ;70
   db   0, -2, -3, -5, -7, -9,-10,-12,-14,-16,-17,-19,-21,-22,-24,-26 ;80
   db -27,-29,-30,-32,-34,-35,-37,-38,-40,-41,-42,-44,-45,-47,-48,-49 ;90
   db -50,-52,-53,-54,-55,-56,-57,-58,-59,-60,-61,-62,-63,-64,-64,-65 ;a0
   db -66,-66,-67,-68,-68,-69,-69,-69,-70,-70,-70,-71,-71,-71,-71,-71 ;b0
   db -71,-71,-71,-71,-71,-71,-70,-70,-70,-69,-69,-69,-68,-68,-67,-66 ;c0
   db -66,-65,-64,-64,-63,-62,-61,-60,-59,-58,-57,-56,-55,-54,-53,-52 ;d0
   db -50,-49,-48,-47,-45,-44,-42,-41,-40,-38,-37,-35,-34,-32,-30,-29 ;e0
   db -27,-26,-24,-22,-21,-19,-17,-16,-14,-12,-10, -9, -7, -5, -3, -2 ;f0

.ENDLITERAL
DTM_APIINT_END:

; End of File DTM.asm
