// ///////////////////////////////////////////////////////////////////
#include <Global.h>
#include <URT.h>          // part specific constants and macros
// ///////////////////////////////////////////////////////////////////
/*
BYTE  UART_bReadRxStatus(void)

UART_RX_REG_FULL Data register contains unread data
UART_RX_PARITY_ERROR  Parity error since last time status read
UART_RX_OVERRUN_ERROR  Data has been over written to buffer register
UART_RX_FRAMING_ERROR  Stop bit is low
UART_RX_ERROR  All Error bits OR’ed together
*/

// ///////////////////////////////////////////////////////////////////
volatile byte RxFlag, RxIdx, RxLen;
volatile byte RxBuf[UART_BUF_SIZE];
//volatile byte RxRec[UART_BUF_SIZE];
// ///////////////////////////////////////////////////////////////////
void ISRC_RxReady(void)		// receive data ready
{ byte ss;
	
//	LED_ON;
	M8C_DisableGInt;       	// enable global interrupts
	EnterUART();	
	if( (UART_bReadRxStatus()&UART_RX_REG_FULL) && !(UART_bReadRxStatus()&UART_RX_ERROR) )
	{	ss= UART_bReadRxData();
		if( ss==STX ) 			
		{	RxIdx= 0;					// if start text: clear pointer
		}
		else if( ss>=' ' )				// if text data
		{	RxBuf[RxIdx++]= ss;			//	 set buffer
			if( RxIdx>=UART_BUF_SIZE-1 )  // if exceed data
			{	RxIdx= UART_BUF_SIZE-1;
				RxBuf[RxIdx]= 0;		  // complete buffer
			//	strncpy( (char*)RxRec, (char*)RxBuf, RxIdx );
				RxLen= RxIdx;
				RxIdx= 0;
				RxFlag= 1;
				ExitUART();	
		}	}	 
		else if( ss==CR ) 				// if end of text
		{   if( RxIdx )
			{	if( RxIdx>=UART_BUF_SIZE-1 ) RxIdx= UART_BUF_SIZE-1;
				RxBuf[RxIdx]= 0;		  // complete buffer
			//	strncpy( (char*)RxRec, (char*)RxBuf, RxIdx );
				RxLen= RxIdx;
				RxIdx= 0;
				RxFlag= 1;
				ExitUART();	
	}	}	}
	M8C_EnableGInt;       	// enable global interrupts
//	LED_OFF;
}

// ///////////////////////////////////////////////////////////////////
void UART_Init(void)
{   UART_Start(UART_PARITY_EVEN);     // Parity Even  
    UART_EnableInt();
	UART_IntCntl(UART_ENABLE_RX_INT); // Enable RX interrupts
    M8C_EnableGInt ;                  // Turn on interrupts  
}
// ///////////////////////////////////////////////////////////////////
void UART_StopEx(void)
{   M8C_DisableGInt ;               // Turn off interrupts  
	UART_Stop();
}

// ///////////////////////////////////////////////////////////////////
byte UART_Send( byte str[] )
{ byte len;
	
	EnterUART();	
	len= strlen( str );
	if( len<1 ) return(0);		// length must greater than 1 
	if( len>UART_BUF_SIZE-1 ) len= UART_BUF_SIZE-1;
	str[len]= 0;				// must have null
	UART_PutChar(STX);			// write STX
	UART_PutString( str );		// write record
	UART_PutChar(CR);			// write CR code
	WaitMs(5);
	ExitUART();	
	return(len); 					
}

// ///////////////////////////////////////////////////////////////////
byte UART_CSend( const char str[] )
{ byte ss;
	cstrcpy( (char*)RxBuf, str );
	ss= UART_Send( (char*)RxBuf );
	return(ss);
}	 

// ///////////////////////////////////////////////////////////////////
byte UART_Receive( byte rec[], byte siz )
{ byte len;
	if( !RxFlag ) return 0;
	if( siz>=UART_BUF_SIZE-1 ) siz= UART_BUF_SIZE-1;  
	len= RxLen;
	RxLen= 0;
	if( siz<len ) len= siz;
	strncpy( rec, (char*)RxBuf, len );
	RxFlag= 0;
	return len;  
}
// ///////////////////////////////////////////////////////////////////
